import gleam/http.{Delete, Get, Post}
import gleam/list
import gleam/string_tree
import wisp.{type Request, type Response}

pub const cookie_name = "pluralkit_token"

pub fn login(req: Request) -> Response {
  case req.method {
    Get -> get_login(req)
    Post -> post_login(req)
    Delete -> delete_login(req)
    _ -> wisp.method_not_allowed([Get, Post, Delete])
  }
}

fn get_login(_req: Request) -> Response {
  [
    "<p>Hi, need that one's PluralKit token! <3</p>",
    "<form action='/login' method='post'>",
    "  <label for='token'><b>PluralKit Token</b></label>",
    "  <input type='string' name='token' id='token' />",
    "  <input type='submit' value='Login'/>", "</form>",
  ]
  |> string_tree.from_strings
  |> wisp.html_response(200)
}

fn post_login(req: Request) -> Response {
  use formdata <- wisp.require_form(req)

  let resp = wisp.redirect("/")
  case list.key_find(formdata.values, "token") {
    Ok(token) ->
      wisp.set_cookie(
        resp,
        req,
        cookie_name,
        token,
        wisp.Signed,
        60 * 60 * 24 * 365,
      )
    Error(_) ->
      wisp.set_header(resp, "x-servfail", "token not found in formdata")
  }
}

fn delete_login(req: Request) -> Response {
  let resp = wisp.redirect("/session")
  case wisp.get_cookie(req, cookie_name, wisp.Signed) {
    Ok(value) -> wisp.set_cookie(resp, req, cookie_name, value, wisp.Signed, 0)
    Error(_) -> resp
  }
}
